@@ -1,12 +1,66 @@
-from typing import List, Optional
+from typing import List, Optional, Dict, Any
 
 import io, pyarrow as pa, requests, sys
 
+from graphistry.privacy import Mode, Privacy, ModeAction
+
+from .client_session import ClientSession
 from .ArrowFileUploader import ArrowFileUploader
+
+from .exceptions import TokenExpireException
+from .validate.validate_encodings import validate_encodings
+from .utils.requests import log_requests_error
 from .util import setup_logger
 logger = setup_logger(__name__)
 
 class ArrowUploader:
+
+    def __init__(
+        self,
+        server_base_path: str = "http://nginx",
+        view_base_path: str = "http://localhost",
+        name: Optional[str] = None,
+        description: Optional[str] = None,
+        edges: Optional[pa.Table] = None,
+        nodes: Optional[pa.Table] = None,
+        node_encodings: Optional[Dict[str, Any]] = None,
+        edge_encodings: Optional[Dict[str, Any]] = None,
+        token: Optional[str] = None,
+        dataset_id: Optional[str] = None,
+        nodes_file_id: Optional[str] = None,
+        edges_file_id: Optional[str] = None,
+        metadata: Optional[Dict[str, Any]] = None,
+        certificate_validation: bool = True,
+        org_name: Optional[str] = None,
+        client_session: Optional[ClientSession] = None,
+    ) -> None:
+        # NOTE: The global is used in standalone tests
+        # in pygraphistry.py the client session is set from the caller
+        if client_session is None:
+            from .pygraphistry import PyGraphistry
+            client_session = PyGraphistry.session
+
+        self._client_session = client_session
+        
+        self.__name = name
+        self.__description = description
+        self.__server_base_path = server_base_path
+        self.__view_base_path = view_base_path
+        self.__token = token
+        self.__dataset_id = dataset_id
+        self.__nodes_file_id = nodes_file_id
+        self.__edges_file_id = edges_file_id
+        self.__edges = edges
+        self.__nodes = nodes
+        self.__node_encodings = node_encodings
+        self.__edge_encodings = edge_encodings
+        self.__metadata = metadata
+        self.__certificate_validation = certificate_validation
+        self.__org_name = org_name or self._client_session.org_name
+
+        self.sso_auth_url: Optional[str] = None
+
+        logger.debug("2. @ArrowUploader.__init__: After set self.org_name: %s, self.__org_name : %s", self.org_name, self.__org_name)
     
     @property
     def token(self) -> str:
@@ -15,7 +69,7 @@ class ArrowUploader:
         return self.__token
 
     @token.setter
-    def token(self, token: str):
+    def token(self, token: Optional[str]):
         self.__token = token
 
     @property
@@ -24,7 +78,7 @@ class ArrowUploader:
 
     @org_name.setter
     def org_name(self, org_name: str) -> None:
-        self.__org_name: Optional[str] = org_name
+        self.__org_name = org_name
 
     @property
     def dataset_id(self) -> str:
@@ -36,6 +90,26 @@ class ArrowUploader:
     def dataset_id(self, dataset_id: str):
         self.__dataset_id = dataset_id
 
+    @property
+    def nodes_file_id(self) -> str:
+        if self.__nodes_file_id is None:
+            raise Exception("Must first create a node file")
+        return self.__nodes_file_id
+
+    @nodes_file_id.setter
+    def nodes_file_id(self, nodes_file_id: str):
+        self.__nodes_file_id = nodes_file_id
+
+    @property
+    def edges_file_id(self) -> str:
+        if self.__edges_file_id is None:
+            raise Exception("Must first create an edges file")
+        return self.__edges_file_id
+
+    @edges_file_id.setter
+    def edges_file_id(self, edges_file_id: str):
+        self.__edges_file_id = edges_file_id
+
     @property
     def server_base_path(self) -> str:
         return self.__server_base_path
@@ -53,19 +127,19 @@ class ArrowUploader:
         self.__view_base_path = view_base_path
 
     @property
-    def edges(self) -> pa.Table:
+    def edges(self) -> Optional[pa.Table]:
         return self.__edges
 
     @edges.setter
-    def edges(self, edges: pa.Table):
+    def edges(self, edges: Optional[pa.Table]):
         self.__edges = edges
 
     @property
-    def nodes(self) -> pa.Table:
+    def nodes(self) -> Optional[pa.Table]:
         return self.__nodes
 
     @nodes.setter
-    def nodes(self, nodes: pa.Table):
+    def nodes(self, nodes: Optional[pa.Table]):
         self.__nodes = nodes
 
     @property
@@ -150,44 +224,6 @@ class ArrowUploader:
     ########################################################################3
 
 
-    def __init__(self, 
-            server_base_path='http://nginx', view_base_path='http://localhost',
-            name = None,
-            description = None,
-            edges = None, nodes = None,
-            node_encodings = None, edge_encodings = None,
-            token = None, dataset_id = None,
-            metadata = None,
-            certificate_validation = True, 
-            org_name: Optional[str] = None):
-
-        self.__name = name
-        self.__description = description
-        self.__server_base_path = server_base_path
-        self.__view_base_path = view_base_path
-        self.__token = token
-        self.__dataset_id = dataset_id
-        self.__edges = edges
-        self.__nodes = nodes
-        self.__node_encodings = node_encodings
-        self.__edge_encodings = edge_encodings
-        self.__metadata = metadata
-        self.__certificate_validation = certificate_validation
-        self.__org_name = org_name if org_name else None
-
-        if org_name:
-            self.__org_name = org_name
-        else:
-            # check current org_name
-            from .pygraphistry import PyGraphistry
-            if 'org_name' in PyGraphistry._config:
-                logger.debug("@ArrowUploader.__init__: There is an org_name : {}".format(PyGraphistry._config['org_name']))
-                self.__org_name = PyGraphistry._config['org_name']
-            else:
-                self.__org_name = None
-
-        logger.debug("2. @ArrowUploader.__init__: After set self.org_name: {}, self.__org_name : {}".format(self.org_name, self.__org_name))
-
 
     def login(self, username, password, org_name=None):
         # base_path = self.server_base_path
@@ -200,10 +236,11 @@ class ArrowUploader:
             f'{self.server_base_path}/api-token-auth/',
             verify=self.certificate_validation,
             json=json_data)
+        log_requests_error(out)
 
         return self._handle_login_response(out, org_name)
 
-    def pkey_login(self, personal_key_id, personal_key_secret, org_name=None):
+    def pkey_login(self, personal_key_id: str, personal_key_secret: str, org_name: Optional[str] = None) -> 'ArrowUploader':
         # json_data = {'personal_key_id': personal_key_id, 'personal_key_secret': personal_key}
         json_data = {}
         if org_name:
@@ -217,9 +254,10 @@ class ArrowUploader:
             url,
             verify=self.certificate_validation,
             json=json_data, headers=headers)
+        log_requests_error(out)
         return self._handle_login_response(out, org_name)
 
-    def _handle_login_response(self, out, org_name):
+    def _handle_login_response(self, out: requests.Response, org_name: Optional[str]) -> 'ArrowUploader':
         from .pygraphistry import PyGraphistry
         json_response = None
         try:
@@ -248,12 +286,12 @@ class ArrowUploader:
                     raise Exception("You are not authorized or not a member of {}".format(org_name))
 
             if logged_in_org_name is None and org_name is None:
-                if 'org_name' in PyGraphistry._config:
-                    del PyGraphistry._config['org_name']
+                if PyGraphistry.session.org_name is not None:
+                    PyGraphistry.session.org_name = None
             else:
-                if org_name in PyGraphistry._config:
-                    logger.debug("@ArrowUploder, handle login reponse, org_name: {}".format(PyGraphistry._config['org_name']))
-                PyGraphistry._config['org_name'] = logged_in_org_name 
+                if PyGraphistry.session.org_name is not None:
+                    logger.debug("@ArrowUploder, handle login reponse, org_name: %s", PyGraphistry.session.org_name)
+                PyGraphistry.session.org_name = logged_in_org_name 
                 # PyGraphistry.org_name(logged_in_org_name)
         except Exception:
             logger.error('Error: %s', out, exc_info=True)
@@ -263,7 +301,7 @@ class ArrowUploader:
 
         return self
 
-    def sso_login(self, org_name=None, idp_name=None):
+    def sso_login(self, org_name: Optional[str] = None, idp_name: Optional[str] = None) -> 'ArrowUploader':
         """
         Koa, 04 May 2022    Get SSO login auth_url or token
         """
@@ -285,17 +323,19 @@ class ArrowUploader:
             url, data={'client-type': 'pygraphistry'},
             verify=self.certificate_validation
         )
-        # print(out.text)
+        log_requests_error(out)
+
         json_response = None
         try:
+            logger.debug("@ArrowUploader.sso_login, out.text: %s", out.text)
             json_response = out.json()
-            logger.debug("@ArrowUploader.sso_login, json_response: {}".format(json_response))
-            self.token = None
+            logger.debug("@ArrowUploader.sso_login, json_response: %s", json_response)
+            self.token = None  # type: ignore[assignment]
             if not ('status' in json_response):
                 raise Exception(out.text)
             else:
                 if json_response['status'] == 'OK':
-                    logger.debug("@ArrowUploader.sso_login, json_data : {}".format(json_response['data']))
+                    logger.debug("@ArrowUploader.sso_login, json_data : %s", json_response['data'])
                     if 'state' in json_response['data']:
                         self.sso_state = json_response['data']['state']
                         self.sso_auth_url = json_response['data']['auth_url']
@@ -306,6 +346,7 @@ class ArrowUploader:
 
         except Exception:
             logger.error('Error: %s', out, exc_info=True)
+            print("\nThere is error with the SSO login, please check your SSO and IDP configuration")
             raise
             
         return self
@@ -322,6 +363,7 @@ class ArrowUploader:
             f'{base_path}/api/v2/o/sso/oidc/jwt/{state}/',
             verify=self.certificate_validation
         )
+        log_requests_error(out)
         json_response = None
         try:
             json_response = out.json()
@@ -334,12 +376,12 @@ class ArrowUploader:
                     if 'token' in json_response['data']:
                         self.token = json_response['data']['token']
                     if 'active_organization' in json_response['data']:
-                        logger.debug("@ArrowUploader.sso_get_token, org_name: {}".format(json_response['data']['active_organization']['slug']))
+                        logger.debug("@ArrowUploader.sso_get_token, org_name: %s", json_response['data']['active_organization']['slug'])
                         self.org_name = json_response['data']['active_organization']['slug']
 
-        except Exception:
-            logger.error('Error: %s', out, exc_info=True)
-            # raise
+        except Exception as e:
+            logger.error('Unexpected SSO authentication error: %s', out, exc_info=True)
+            raise e
             
         return self
 
@@ -350,17 +392,22 @@ class ArrowUploader:
 
         base_path = self.server_base_path
         out = requests.post(
-            f'{base_path}/api-token-refresh/',
+            f'{base_path}/api/v2/auth/token/refresh',
             verify=self.certificate_validation,
             json={'token': token})
+        log_requests_error(out)
         json_response = None
         try:
             json_response = out.json()
             if not ('token' in json_response):
+                if (
+                    "non_field_errors" in json_response and "Token has expired." in json_response["non_field_errors"]
+                ):
+                    raise TokenExpireException(out.text)
                 raise Exception(out.text)
-        except Exception:
+        except Exception as e:
             logger.error('Error: %s', out, exc_info=True)
-            raise Exception(out.text)
+            raise e
             
         self.token = out.json()['token']        
         return self
@@ -374,19 +421,26 @@ class ArrowUploader:
             f'{base_path}/api-token-verify/',
             verify=self.certificate_validation,
             json={'token': token})
+        log_requests_error(out)
         return out.status_code == requests.codes.ok
 
-    def create_dataset(self, json):  # noqa: F811
+    def create_dataset(self, json, validate: bool = True):  # noqa: F811
+        if validate:
+            validate_encodings(
+                json.get('node_encodings', {}),
+                json.get('edge_encodings', {}),
+                self.nodes.column_names if self.nodes is not None else None,
+                self.edges.column_names if self.edges is not None else None)
         tok = self.token
         if self.org_name: 
             json['org_name'] = self.org_name
-        logger.debug("@ArrowUploder create_dataset json: {}".format(json))
+        logger.debug("@ArrowUploder create_dataset json: %s", json)
         res = requests.post(
             self.server_base_path + '/api/v2/upload/datasets/',
             verify=self.certificate_validation,
             headers={'Authorization': f'Bearer {tok}'},
             json=json)
-             
+        log_requests_error(res)
         try: 
             out = res.json()
             if not out['success']:
@@ -484,22 +538,31 @@ class ArrowUploader:
         return encodings
 
 
-    def post(self, as_files: bool = True, memoize: bool = True):
+    def post(
+        self,
+        as_files: bool = True,
+        memoize: bool = True,
+        validate: bool = True,
+        erase_files_on_fail: bool = True
+    ) -> 'ArrowUploader':
         """
         Note: likely want to pair with self.maybe_post_share_link(g)
         """
-        logger.debug("@ArrowUploader.post, self.org_name : {}".format(self.org_name))
+        logger.debug("@ArrowUploader.post, self.org_name : %s", self.org_name)
         if as_files:
 
             file_uploader = ArrowFileUploader(self)
             file_opts = {'name': self.name + ' edges'}
             if self.org_name:
                 file_opts['org_name'] = self.org_name
+            upload_url_opts = 'erase=true' if erase_files_on_fail else 'erase=false'
 
-            e_file_id, _ = file_uploader.create_and_post_file(self.edges, file_opts=file_opts)
+            e_file_id, _ = file_uploader.create_and_post_file(self.edges, file_opts=file_opts, upload_url_opts=upload_url_opts)
+            self.edges_file_id = e_file_id
 
             if not (self.nodes is None):
-                n_file_id, _ = file_uploader.create_and_post_file(self.nodes, file_opts=file_opts)
+                n_file_id, _ = file_uploader.create_and_post_file(self.nodes, file_opts=file_opts, upload_url_opts=upload_url_opts)
+                self.nodes_file_id = n_file_id
 
             self.create_dataset({
                 "node_encodings": self.node_encodings,
@@ -509,7 +572,7 @@ class ArrowUploader:
                 "description": self.description,
                 "edge_files": [ e_file_id ],
                 **({"node_files": [ n_file_id ] if not (self.nodes is None) else []})
-            })
+            }, validate)
 
         else:
 
@@ -519,13 +582,13 @@ class ArrowUploader:
                 "metadata": self.metadata,
                 "name": self.name,
                 "description": self.description
-            })
-            
+            }, validate)
+
             self.post_edges_arrow()
-            
+
             if not (self.nodes is None):
                 self.post_nodes_arrow()
-        
+
         return self
 
 
@@ -534,32 +597,31 @@ class ArrowUploader:
 
     def cascade_privacy_settings(
         self,
-        mode: Optional[str] = None,
+        mode: Optional[Mode] = None,
         notify: Optional[bool] = None,
-        invited_users: Optional[List] = None,
-        mode_action: Optional[str] = None,
+        invited_users: Optional[List[str]] = None,
+        mode_action: Optional[ModeAction] = None,
         message: Optional[str] = None
     ):
         """
         Cascade:
             - local (passed in)
-            - global
+            - session
             - hard-coded
         """
 
-        from .pygraphistry import PyGraphistry
-        global_privacy = PyGraphistry._config['privacy']
-        if global_privacy is not None:
+        session_privacy = self._client_session.privacy
+        if session_privacy is not None:
             if mode is None:
-                mode = global_privacy['mode']
+                mode = session_privacy.get('mode')
             if notify is None:
-                notify = global_privacy['notify']
+                notify = session_privacy.get('notify')
             if invited_users is None:
-                invited_users = global_privacy['invited_users']
+                invited_users = session_privacy.get('invited_users')
             if mode_action is None:
-                mode_action = global_privacy['mode_action']
+                mode_action = session_privacy.get('mode_action')
             if message is None:
-                message = global_privacy['message']
+                message = session_privacy.get('message')
 
         if mode is None:
             mode = 'private'
@@ -582,7 +644,7 @@ class ArrowUploader:
         self,
         obj_pk: str,
         obj_type: str = 'dataset',
-        privacy: Optional[dict] = None
+        privacy: Optional[Privacy] = None
     ):
         """
         Set sharing settings. Any settings not passed here will cascade from PyGraphistry or defaults
@@ -605,6 +667,7 @@ class ArrowUploader:
                 'mode_action': mode_action,
                 'message': message
             })
+        log_requests_error(res)
 
         if res.status_code != requests.codes.ok:
             logger.error('Failed setting sharing status (code %s): %s', res.status_code, res.text, exc_info=True)
@@ -632,12 +695,12 @@ class ArrowUploader:
     ###########################################
 
 
-    def post_edges_arrow(self, arr=None, opts=''):
+    def post_edges_arrow(self, arr: Optional[pa.Table] = None, opts=''):
         if arr is None:
             arr = self.edges
         return self.post_arrow(arr, 'edges', opts) 
 
-    def post_nodes_arrow(self, arr=None, opts=''):
+    def post_nodes_arrow(self, arr: Optional[pa.Table] = None, opts=''):
         if arr is None:
             arr = self.nodes
         return self.post_arrow(arr, 'nodes', opts) 
@@ -654,9 +717,9 @@ class ArrowUploader:
                 raise Exception('No success indicator in server response')
             return out
         except requests.exceptions.HTTPError as e:
-            logger.error('Failed to post arrow to %s (%s)', sub_path, e.request.url, exc_info=True)
+            logger.error('Failed to post arrow to %s (%s)', sub_path, "{}/{}{}".format(self.server_base_path, sub_path, f"?{opts}" if len(opts) > 0 else ""), exc_info=True)
             logger.error('%s', e)
-            logger.error('%s', e.response.text)
+            logger.error('%s', e.response.text if e.response else None)
             raise e
         except Exception as e:
             logger.error('Failed to post arrow to %s', sub_path, exc_info=True)
@@ -675,8 +738,11 @@ class ArrowUploader:
             verify=self.certificate_validation,
             headers={'Authorization': f'Bearer {tok}'},
             data=buf)
-                    
+        log_requests_error(resp)
+
         if resp.status_code != requests.codes.ok:
+
+            log_requests_error(resp)
             resp.raise_for_status()
 
         return resp
@@ -689,9 +755,9 @@ class ArrowUploader:
             Skip if never called .privacy()
             Return True/False based on whether called
         """
-        from .pygraphistry import PyGraphistry
-        logger.debug('Privacy: global (%s), local (%s)', PyGraphistry._config['privacy'] or 'None', g._privacy or 'None')
-        if PyGraphistry._config['privacy'] is not None or g._privacy is not None:
+        session_privacy = self._client_session.privacy
+        logger.debug('Privacy: global (%s), local (%s)', session_privacy or 'None', g._privacy or 'None')
+        if session_privacy is not None or g._privacy is not None:
             self.post_share_link(self.dataset_id, 'dataset', g._privacy)
             return True
 
@@ -737,6 +803,7 @@ class ArrowUploader:
                 verify=self.certificate_validation,
                 headers={'Authorization': f'Bearer {tok}'},
                 data=file.read()).json()
+            log_requests_error(out)
             if not out['success']:
                 raise Exception(out)
             
