@@ -1,10 +1,27 @@
-import logging, numpy as np, pandas as pd
-from typing import Any, List, Union, TYPE_CHECKING
+import numpy as np
+import pandas as pd
+from typing import Any, Dict, List, Optional, Union
 from typing_extensions import Literal
+from inspect import getmodule
 
-from graphistry.Engine import Engine
+from graphistry.Engine import Engine, EngineAbstract
 from graphistry.Plottable import Plottable
-from .chain import chain as chain_base
+from graphistry.util import setup_logger
+from graphistry.utils.json import JSONVal
+from .ast import ASTObject
+from .chain import Chain, chain as chain_base
+from .chain_let import chain_let as chain_let_base
+from .gfql_unified import gfql as gfql_base
+from .chain_remote import (
+    chain_remote as chain_remote_base,
+    chain_remote_shape as chain_remote_shape_base
+)
+from .python_remote import (
+    python_remote_g as python_remote_g_base,
+    python_remote_table as python_remote_table_base,
+    python_remote_json as python_remote_json_base
+)
+from graphistry.models.compute.chain_remote import OutputTypeGraph, FormatType
 from .collapse import collapse_by
 from .hop import hop as hop_base
 from .filter_by_dict import (
@@ -12,22 +29,75 @@ from .filter_by_dict import (
     filter_nodes_by_dict as filter_nodes_by_dict_base
 )
 
-logger = logging.getLogger("compute")
-
-if TYPE_CHECKING:
-    MIXIN_BASE = Plottable
-else:
-    MIXIN_BASE = object
+logger = setup_logger(__name__)
 
 
-class ComputeMixin(MIXIN_BASE):
-    def __init__(self, *args, **kwargs):
-        pass
+class ComputeMixin(Plottable):
+    
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
+    
+    def to_cudf(self) -> 'Plottable':
+        """
+        Convert to GPU mode by converting any defined nodes and edges to cudf dataframes
+
+        When nodes or edges are already cudf dataframes, they are left as is
+
+        :param g: Graphistry object
+        :type g: Plottable
+
+        :return: Graphistry object
+
+        """
+
+        import cudf
+
+        g = self.bind()
+        if g._edges is not None:
+            if not isinstance(g._edges, cudf.DataFrame):
+                if isinstance(g._edges, pd.DataFrame):
+                    g = g.edges(cudf.from_pandas(g._edges))
+                else:
+                    raise ValueError('Expected edges to be pandas, got: {}'.format(type(g._edges)))
+                
+        if g._nodes is not None:
+            if not isinstance(g._nodes, cudf.DataFrame):
+                if isinstance(g._nodes, pd.DataFrame):
+                    g = g.nodes(cudf.from_pandas(g._nodes))
+                else:
+                    raise ValueError('Expected nodes to be pandas, got: {}'.format(type(g._nodes)))
+        
+        return g
+                
+    def to_pandas(self) -> 'Plottable':
+        """
+        Convert to CPU mode by converting any defined nodes and edges to pandas dataframes
+
+        When nodes or edges are already pandas dataframes, they are left as is"""
+
+        g = self.bind()
+        if g._edges is not None:
+            if not isinstance(g._edges, pd.DataFrame):
+                import cudf
+                if isinstance(g._edges, cudf.DataFrame):
+                    g = g.edges(g._edges.to_pandas())
+                else:
+                    raise ValueError('Expected edges to be cudf, got: {}'.format(type(g._edges)))
+                
+        if g._nodes is not None:
+            if not isinstance(g._nodes, pd.DataFrame):
+                import cudf
+                if isinstance(g._nodes, cudf.DataFrame):
+                    g = g.nodes(g._nodes.to_pandas())
+                else:
+                    raise ValueError('Expected nodes to be cudf, got: {}'.format(type(g._nodes)))
+        
+        return g
 
     def materialize_nodes(
         self,
         reuse: bool = True,
-        engine: Union[Engine, Literal['auto']] = "auto"
+        engine: Union[EngineAbstract, str] = EngineAbstract.AUTO
     ) -> "Plottable":
         """
         Generate g._nodes based on g._edges
@@ -49,17 +119,13 @@ class ComputeMixin(MIXIN_BASE):
                 print(g2._nodes)  # pd.DataFrame
 
         """
-        g = self
-        if g._edges is None:
-            raise ValueError("Missing edges")
-        if g._source is None or g._destination is None:
-            raise ValueError(
-                "Missing source/destination bindings; set via .bind() or .edges()"
-            )
-        if len(g._edges) == 0:
-            return g
-        # TODO use built-ins for igraph/nx/...
 
+        if isinstance(engine, str):
+            engine = EngineAbstract(engine)
+
+        g = self
+
+        # Check reuse first - if we have nodes and reuse is True, just return
         if reuse:
             if g._nodes is not None and len(g._nodes) > 0:
                 if g._node is None:
@@ -70,22 +136,50 @@ class ComputeMixin(MIXIN_BASE):
                 else:
                     return g
 
+        # Only check for edges if we actually need to materialize
+        if g._edges is None:
+            # If no edges but we have nodes via reuse, that's OK
+            if reuse and g._nodes is not None and len(g._nodes) > 0:
+                return g
+            raise ValueError("Missing edges")
+        if g._source is None or g._destination is None:
+            raise ValueError(
+                "Missing source/destination bindings; set via .bind() or .edges()"
+            )
+        if len(g._edges) == 0:
+            return g
+        # TODO use built-ins for igraph/nx/...
+
         node_id = g._node if g._node is not None else "id"
-        if engine == 'auto':
+        engine_concrete : Engine
+        if engine == EngineAbstract.AUTO:
             if isinstance(g._edges, pd.DataFrame):
-                engine = Engine.PANDAS
+                engine_concrete = Engine.PANDAS
             else:
+
+                def raiser(df: Any):
+                    raise ValueError('Could not determine engine for edges, expected pandas or cudf dataframe, got: {}'.format(type(df)))
+
                 try:
-                    import cudf
-                    if isinstance(g._edges, cudf.DataFrame):
-                        engine = Engine.CUDF
-                except ImportError:
-                    pass
-            if engine == 'auto':
-                raise ValueError('Could not determine engine for edges, expected pandas or cudf dataframe, got: {}'.format(type(g._edges)))
-        if engine == Engine.PANDAS:
+                    if 'cudf' in str(getmodule(g._edges)):
+                        import cudf
+                        if isinstance(g._edges, cudf.DataFrame):
+                            engine_concrete = Engine.CUDF
+                        else:
+                            raiser(g._edges)
+                    else:
+                        raiser(g._edges)
+                except ImportError as e:
+                    raise e
+                except Exception:
+                    raiser(g._edges)
+
+        else:
+            engine_concrete = Engine(engine.value)
+
+        if engine_concrete == Engine.PANDAS:
             concat_df = pd.concat([g._edges[g._source], g._edges[g._destination]])
-        elif engine == Engine.CUDF:
+        elif engine_concrete == Engine.CUDF:
             import cudf
             if isinstance(g._edges, cudf.DataFrame):
                 edges_gdf = g._edges
@@ -95,7 +189,7 @@ class ComputeMixin(MIXIN_BASE):
                 raise ValueError('Unexpected edges type; convert edges to cudf.DataFrame')
             concat_df = cudf.concat([edges_gdf[g._source].rename(node_id), edges_gdf[g._destination].rename(node_id)])
         else:
-            raise ValueError('Expected engine to be pandas or cudf, got: {}'.format(engine))
+            raise ValueError('Expected engine to be pandas or cudf, got: {}'.format(engine_concrete))
         nodes_df = concat_df.rename(node_id).drop_duplicates().to_frame().reset_index(drop=True)
         return g.nodes(nodes_df, node_id)
 
@@ -113,8 +207,9 @@ class ComputeMixin(MIXIN_BASE):
         nodes_df = g_nodes._nodes[
             [c for c in g_nodes._nodes.columns if c != col]
         ].merge(in_degree_df, how="left", on=g._node)
-        nodes_df[col].fillna(0, inplace=True)
-        nodes_df[col] = nodes_df[col].astype("int32")
+        nodes_df = nodes_df.assign(**{
+            col: nodes_df[col].fillna(0).astype("int32")
+        })
         return g.nodes(nodes_df, g_nodes._node)
 
     def get_outdegrees(self, col: str = "degree_out"):
@@ -347,6 +442,9 @@ class ComputeMixin(MIXIN_BASE):
         :param node: start `node` to begin traversal
         :param attribute: the given `attribute` to collapse over within `column`
         :param column: the `column` of nodes DataFrame that contains `attribute` to collapse over
+        :param self_edges: whether to include self edges in the collapsed graph
+        :param unwrap: whether to unwrap the collapsed graph into a single node
+        :param verbose: whether to print out collapse summary information
 
         :returns:A new Graphistry instance with nodes and edges DataFrame containing collapsed nodes and edges given by column attribute -- nodes and edges DataFrames contain six new columns `collapse_{node | edges}` and `final_{node | edges}`, while original (node, src, dst) columns are left untouched
         :rtype: Plottable
@@ -378,5 +476,130 @@ class ComputeMixin(MIXIN_BASE):
     filter_edges_by_dict.__doc__ = filter_edges_by_dict_base.__doc__
 
     def chain(self, *args, **kwargs):
+        """
+        .. deprecated:: 2.XX.X
+           Use :meth:`gfql` instead for a unified API that supports both chains and DAGs.
+        """
+        import warnings
+        warnings.warn(
+            "chain() is deprecated. Use gfql() instead for a unified API.",
+            DeprecationWarning,
+            stacklevel=2
+        )
         return chain_base(self, *args, **kwargs)
-    chain.__doc__ = chain_base.__doc__
+    # Preserve original docstring after deprecation notice
+    chain.__doc__ = (chain.__doc__ or "") + "\n\n" + (chain_base.__doc__ or "")
+
+    # chain_let removed from public API - use gfql() instead
+    # (chain_let_base still available internally for gfql dispatch)
+    
+    # Commented out to remove from public API - use gfql() instead
+    # def chain_let(self, *args, **kwargs):
+    #     """Execute a DAG of named graph operations with dependency resolution."""
+    #     return chain_let_base(self, *args, **kwargs)
+    # chain_let.__doc__ = chain_let_base.__doc__
+    
+    def gfql(self, *args, **kwargs):
+        return gfql_base(self, *args, **kwargs)
+    gfql.__doc__ = gfql_base.__doc__
+
+    def chain_remote(self, *args, **kwargs) -> Plottable:
+        """
+        .. deprecated:: 2.XX.X
+           Use :meth:`gfql_remote` instead for a unified API that supports both chains and DAGs.
+        """
+        import warnings
+        warnings.warn(
+            "chain_remote() is deprecated. Use gfql_remote() instead for a unified API.",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        return chain_remote_base(self, *args, **kwargs)
+    # Preserve original docstring after deprecation notice
+    chain_remote.__doc__ = (chain_remote.__doc__ or "") + "\n\n" + (chain_remote_base.__doc__ or "")
+
+    def chain_remote_shape(self, *args, **kwargs) -> pd.DataFrame:
+        """
+        .. deprecated:: 2.XX.X
+           Use :meth:`gfql_remote_shape` instead for a unified API that supports both chains and DAGs.
+        """
+        import warnings
+        warnings.warn(
+            "chain_remote_shape() is deprecated. Use gfql_remote_shape() instead for a unified API.",
+            DeprecationWarning,
+            stacklevel=2
+        )
+        return chain_remote_shape_base(self, *args, **kwargs)
+    # Preserve original docstring after deprecation notice
+    chain_remote_shape.__doc__ = (chain_remote_shape.__doc__ or "") + "\n\n" + (chain_remote_shape_base.__doc__ or "")
+
+    def gfql_remote(
+        self,
+        chain: Union[Chain, List[ASTObject], Dict[str, JSONVal]],
+        api_token: Optional[str] = None,
+        dataset_id: Optional[str] = None,
+        output_type: OutputTypeGraph = "all",
+        format: Optional[FormatType] = None,
+        df_export_args: Optional[Dict[str, Any]] = None,
+        node_col_subset: Optional[List[str]] = None,
+        edge_col_subset: Optional[List[str]] = None,
+        engine: Optional[Literal["pandas", "cudf"]] = None,
+        validate: bool = True,
+        persist: bool = False
+    ) -> Plottable:
+        """Run GFQL query remotely.
+
+        This is the remote execution version of :meth:`gfql`. It supports both simple chains
+        and complex DAG patterns with Let bindings, including transformations like hypergraph.
+
+        Example:
+            # Remote hypergraph transformation
+            hg = g.gfql_remote(call('hypergraph', {'entity_types': ['user', 'product']}))
+
+            # Or using typed builder
+            from graphistry.compute import hypergraph
+            hg = g.gfql_remote(hypergraph(entity_types=['user', 'product']))
+
+        See :meth:`chain_remote` for detailed documentation (chain_remote is deprecated).
+        """
+        return chain_remote_base(
+            self, chain, api_token, dataset_id, output_type, format,
+            df_export_args, node_col_subset, edge_col_subset, engine, validate, persist
+        )
+    
+    def gfql_remote_shape(
+        self,
+        chain: Union[Chain, List[ASTObject], Dict[str, JSONVal]],
+        api_token: Optional[str] = None,
+        dataset_id: Optional[str] = None,
+        format: Optional[FormatType] = None,
+        df_export_args: Optional[Dict[str, Any]] = None,
+        node_col_subset: Optional[List[str]] = None,
+        edge_col_subset: Optional[List[str]] = None,
+        engine: Optional[Literal["pandas", "cudf"]] = None,
+        validate: bool = True,
+        persist: bool = False
+    ) -> pd.DataFrame:
+        """Get shape metadata for remote GFQL query execution.
+
+        This is the remote shape version of :meth:`gfql`. Returns metadata about the
+        resulting graph without downloading the full data.
+
+        See :meth:`chain_remote_shape` for detailed documentation (chain_remote_shape is deprecated).
+        """
+        return chain_remote_shape_base(
+            self, chain, api_token, dataset_id, format, df_export_args,
+            node_col_subset, edge_col_subset, engine, validate, persist
+        )
+
+    def python_remote_g(self, *args, **kwargs) -> Any:
+        return python_remote_g_base(self, *args, **kwargs)
+    python_remote_g.__doc__ = python_remote_g_base.__doc__
+
+    def python_remote_table(self, *args, **kwargs) -> Any:
+        return python_remote_table_base(self, *args, **kwargs)
+    python_remote_table.__doc__ = python_remote_table_base.__doc__
+
+    def python_remote_json(self, *args, **kwargs) -> Any:
+        return python_remote_json_base(self, *args, **kwargs)
+    python_remote_json.__doc__ = python_remote_json_base.__doc__
