@@ -1,10 +1,16 @@
 # classes for converting a dataframe or Graphistry Plottable into a DGL
 from collections import Counter
-from typing import Optional, TYPE_CHECKING
+from inspect import getmodule
+from typing import Dict, Optional, TYPE_CHECKING, Tuple, cast, Any
+import warnings
 
 import numpy as np
 import pandas as pd
 
+from graphistry.utils.lazy_import import (
+    lazy_dgl_import,
+    lazy_torch_import_has_dependency
+)
 from . import constants as config
 from .feature_utils import (
     FeatureEngine,
@@ -20,6 +26,7 @@ from .util import setup_logger
 
 
 if TYPE_CHECKING:
+    import scipy
     MIXIN_BASE = FeatureMixin
     try:
         import torch
@@ -33,27 +40,7 @@ else:
     MIXIN_BASE = object
 
 
-def lazy_dgl_import_has_dependency():
-    try:
-        import warnings
-        warnings.filterwarnings('ignore')
-        import dgl  # noqa: F811
-        return True, 'ok', dgl
-    except ModuleNotFoundError as e:
-        return False, e, None
-
-
-def lazy_torch_import_has_dependency():
-    try:
-        import warnings
-        warnings.filterwarnings('ignore')
-        import torch  # noqa: F811
-        return True, 'ok', torch
-    except ModuleNotFoundError as e:
-        return False, e, None
-
-
-logger = setup_logger(name=__name__, verbose=config.VERBOSE)
+logger = setup_logger(name=__name__)
 
 
 
@@ -126,12 +113,26 @@ def reindex_edgelist(df, src, dst):
     """
     srclist = df[src]
     dstlist = df[dst]
-    cnt = Counter(
-        pd.concat([srclist, dstlist], axis=0)
-    )  # can also use pd.Factorize but doesn't order by count, which is satisfying
-    ordered_nodes_dict = {k: i for i, (k, c) in enumerate(cnt.most_common())}
-    df[config.SRC] = df[src].apply(lambda x: ordered_nodes_dict[x])
-    df[config.DST] = df[dst].apply(lambda x: ordered_nodes_dict[x])
+    if isinstance(df, pd.DataFrame):
+        cnt = Counter(
+            pd.concat([srclist, dstlist], axis=0)
+        )  # can also use pd.Factorize but doesn't order by count, which is satisfying
+        ordered_nodes_dict = {k: i for i, (k, c) in enumerate(cnt.most_common())}
+        df[config.SRC] = df[src].apply(lambda x: ordered_nodes_dict[x])
+        df[config.DST] = df[dst].apply(lambda x: ordered_nodes_dict[x])
+    elif 'cudf' in str(getmodule(df)):
+        import cudf
+        if isinstance(df, cudf.DataFrame):
+            cnt = Counter(
+                cudf.concat([srclist, dstlist], axis=0).to_pandas()
+            )
+            ordered_nodes_dict = {k: i for i, (k, c) in enumerate(cnt.most_common())}
+            df[config.SRC] = cudf.Series.from_pandas(df[src].to_pandas().apply(lambda x: ordered_nodes_dict[x]))
+            df[config.DST] = cudf.Series.from_pandas(df[dst].to_pandas().apply(lambda x: ordered_nodes_dict[x]))
+        else:
+            raise ValueError("df must be cudf.DataFrame or pd.DataFrame")
+    else:
+        raise ValueError("df must be cudf.DataFrame or pd.DataFrame")
     return df, ordered_nodes_dict
 
 
@@ -156,6 +157,10 @@ def pandas_to_sparse_adjacency(df, src, dst, weight_col):
         eweight = df[weight_col].values
     
     shape = len(ordered_nodes_dict)
+    if not isinstance(df, pd.DataFrame):
+        if 'cudf' in str(getmodule(df)):
+            warnings.warn("cudf not supported for coo_matrix, converting to pandas")
+            df = df.to_pandas() 
     sp_mat = coo_matrix(
         (eweight, (df[config.SRC], df[config.DST])), shape=(shape, shape)
     )
@@ -166,7 +171,7 @@ def pandas_to_sparse_adjacency(df, src, dst, weight_col):
 
 def pandas_to_dgl_graph(
     df: pd.DataFrame, src: str, dst: str, weight_col: Optional[str] = None, device: str = "cpu"
-):
+) -> Tuple["dgl.DGLGraph", "scipy.sparse.coo_matrix", Dict]:
     """Turns an edge DataFrame with named src and dst nodes, to DGL graph
     :eg
         g, sp_mat, ordered_nodes_dict = pandas_to_sparse_adjacency(df, 'to_node', 'from_node')
@@ -180,7 +185,7 @@ def pandas_to_dgl_graph(
         sp_mat: sparse scipy matrix
         ordered_nodes_dict: dict ordered from most common src and dst nodes
     """
-    _, _, dgl = lazy_dgl_import_has_dependency()  # noqa: F811
+    _, _, dgl = lazy_dgl_import()  # noqa: F811
     sp_mat, ordered_nodes_dict = pandas_to_sparse_adjacency(df, src, dst, weight_col)
     g = dgl.from_scipy(sp_mat, device=device)  # there are other ways too
     logger.info(f"Graph Type: {type(g)}") 
@@ -209,34 +214,32 @@ def get_torch_train_test_mask(n: int, ratio: float = 0.8):
 #######################################################################################################################
 
 
-class DGLGraphMixin(MIXIN_BASE):
+class DGLGraphMixin(FeatureMixin):
     """
         Automagic DGL models from Graphistry Instances.
         
     """
-    def __init__(
-        self,
-    ):
-        """
-        :param train_split: split percent between train and test, set in mask on dgl edata/ndata
-        :param device: Whether to put on cpu or gpu. Can always envoke .to(gpu) on returned DGL graph later, Default 'cpu'
-        """
+    _dgl_graph: Optional[Any]
+    _edges: Any
+    _edge_target : Optional[pd.DataFrame]
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.dgl_initialized = False
 
     def dgl_lazy_init(self, train_split: float = 0.8, device: str = "cpu"):
         """
         Initialize DGL graph lazily
-        :return:
+        :return:  
         """
 
         if not self.dgl_initialized:
-            lazy_dgl_import_has_dependency()
+            lazy_dgl_import()
             lazy_torch_import_has_dependency()
             self.train_split = train_split
             self.device = device
             self._removed_edges_previously = False
-            self.DGL_graph = None
+            self._dgl_graph = None
             self.dgl_initialized = True
 
     def _prune_edge_target(self):
@@ -254,8 +257,8 @@ class DGLGraphMixin(MIXIN_BASE):
         if self._source is None or self._destination is None:
             raise ValueError("Need to have source and destination columns bound, call bind() or edges()")
         
-        if not isinstance(self._edges, pd.DataFrame):  # type: ignore
-            raise ValueError("self._edges for DGLGraphMix must be pd.DataFrame, recieved: %s", type(self._edges))  # type: ignore
+        if 'cudf' not in str(getmodule(self._edges)) and not isinstance(self._edges, pd.DataFrame):
+            raise ValueError("self._edges for DGLGraphMix must be pd.DataFrame/cudf.DataFrame, recieved: %s", type(self._edges))  # type: ignore
         edf: pd.DataFrame = self._edges  # type: ignore
         n_initial = len(edf)
         logger.info(f"Length of edge DataFrame {n_initial}")
@@ -265,12 +268,12 @@ class DGLGraphMixin(MIXIN_BASE):
         # print(f'OG: length: {len(edf)}')
 
         assert (
-            sum(mask) > 2
+            mask.sum() > 2
         ), f"mask slice is (practically) empty, will lead to bad graph, found {sum(mask)}"
-        self._MASK = mask   # type: ignore
-        self._edges = edf[mask]   # type: ignore
+        self._MASK = mask
+        self._edges = edf[mask]
 
-        # print(f'new EDGES: length: {len(self._edges)}')
+        logger.debug(f'new EDGES: length: {len(self._edges)}')
 
         self._prune_edge_target()
         n_final = len(self._edges)
@@ -302,7 +305,7 @@ class DGLGraphMixin(MIXIN_BASE):
                 f"Nodes DataFrame has duplicate entries for column {node_column}"
             )
         # now check that self._entity_to_index is in 1-1 to with self.ndf[node_column]
-        #nodes = self._nodes[node_column]
+        # nodes = self._nodes[node_column]
         res = nodes.isin(self._entity_to_index)
         if res.sum() != len(nodes):
             logger.warning(
@@ -321,7 +324,7 @@ class DGLGraphMixin(MIXIN_BASE):
         if inplace:
             res = self
         else:
-            res = self.bind()
+            res = cast('DGLGraphMixin', self.bind())
 
         if res._node is None:
             res._node = config.IMPLICIT_NODE_ID
@@ -342,13 +345,15 @@ class DGLGraphMixin(MIXIN_BASE):
                 'destination column not set, try running g.bind(destination="my_col") or g.edges(df, destination="my_col")'
             )
 
-        res.DGL_graph, res._adjacency, res._entity_to_index = pandas_to_dgl_graph(
+        res._dgl_graph, res._adjacency, res._entity_to_index = pandas_to_dgl_graph(
             res._edges,
             res._source,
             res._destination,
             weight_col=weight_column,
             device=res.device,
         )
+        if res._entity_to_index is None:
+            raise ValueError("entity_to_index is None, something went wrong")
         res._index_to_entity = {k: v for v, k in res._entity_to_index.items()}
         # this is a sanity check after _remove_edges_not_in_nodes
         res._check_nodes_lineup_with_edges()
@@ -360,21 +365,14 @@ class DGLGraphMixin(MIXIN_BASE):
         feature_engine: FeatureEngine = "auto",
         *args,
         **kwargs,
-        # X: pd.DataFrame,
-        # y: pd.DataFrame,
-        # use_scaler: str = None,
     ):
         logger.info("Running Node Featurization for DGL Graph")
         # logger.debug(f"=*=*=Input shapes are data: {X.shape}, target: {y.shape}")
 
-        X_enc, y_enc, res = res._featurize_or_get_nodes_dataframe_if_X_is_None(
+        X_enc, y_enc, res = res.featurize_or_get_nodes_dataframe_if_X_is_None(
             feature_engine=resolve_feature_engine(feature_engine),
             *args,
             **kwargs
-            # X=X,
-            # y=y,
-            # use_scaler=use_scaler,
-            #feature_engine=resolve_feature_engine(feature_engine),
         )
 
         logger.debug(
@@ -384,7 +382,7 @@ class DGLGraphMixin(MIXIN_BASE):
 
         ndata = convert_to_torch(X_enc, y_enc)
         # add ndata to the graph
-        res.DGL_graph.ndata.update(ndata)
+        res._dgl_graph.ndata.update(ndata)
         res._mask_nodes()
         return res
 
@@ -395,21 +393,13 @@ class DGLGraphMixin(MIXIN_BASE):
         feature_engine: FeatureEngine = "auto",
         *args,
         **kwargs
-        # X: pd.DataFrame,
-        # y: pd.DataFrame,
-        # use_scaler: str = None,
-        # feature_engine: FeatureEngine = "auto",
     ):
         logger.info("Running Edge Featurization for DGL Graph")
 
-        X_enc, y_enc, res = res._featurize_or_get_edges_dataframe_if_X_is_None(
+        X_enc, y_enc, res = res.featurize_or_get_edges_dataframe_if_X_is_None(
             feature_engine=resolve_feature_engine(feature_engine),
             *args,
             **kwargs
-            # X=X,
-            # y=y,
-            # use_scaler=use_scaler,
-            # feature_engine=resolve_feature_engine(feature_engine),
         )
         
         logger.debug(
@@ -418,7 +408,7 @@ class DGLGraphMixin(MIXIN_BASE):
 
         edata = convert_to_torch(X_enc, y_enc)
         # add edata to the graph
-        res.DGL_graph.edata.update(edata)
+        res._dgl_graph.edata.update(edata)
         res._mask_edges()
         return res
 
@@ -432,11 +422,11 @@ class DGLGraphMixin(MIXIN_BASE):
         y_nodes: YSymbolic = None,
         y_edges: YSymbolic = None,
         weight_column: Optional[str] = None,
-        reuse_if_existing=True,
-        featurize_edges =True,
-        use_node_scaler: str = "zscale",
+        reuse_if_existing: bool = True,
+        featurize_edges: bool = True,
+        use_node_scaler: Optional[str] = None,
         use_node_scaler_target: Optional[str] = None,
-        use_edge_scaler: str = "zscale",
+        use_edge_scaler: Optional[str] = None,
         use_edge_scaler_target: Optional[str] = None,
         train_split: float = 0.8,
         device: str = "cpu",
@@ -459,17 +449,16 @@ class DGLGraphMixin(MIXIN_BASE):
         :param weight_column: Optional Weight column if explicit edges table exists with said weights.
                 Otherwise, weight_column is inhereted by UMAP.
         :param train_split: Randomly assigns a train and test mask according to the split value, default 80%.
-        :param use_node_scaler: selects which scaling to use on featurized nodes dataframe. Default `robust`
-        :param use_edge_scaler: selects which scaling to use on featurized edges dataframe. Default `robust`
+        :param use_node_scaler: selects which scaling to use on featurized nodes dataframe. Default None
+        :param use_edge_scaler: selects which scaling to use on featurized edges dataframe. Default None
         :param device: device to run model, default `cpu`, with `gpu` the other choice. Can be handled in outer scope.
         :param inplace: default, False, whether to return Graphistry instance in place or not.
 
         """
-
         if inplace:
             res = self
         else:
-            res = self.bind()
+            res = cast('DGLGraphMixin', self.bind())
 
         res.dgl_lazy_init(train_split=train_split, device=device)
 
@@ -511,7 +500,6 @@ class DGLGraphMixin(MIXIN_BASE):
         res = res._featurize_nodes_to_dgl(
             res,
             **kwargs_nodes
-            #X_nodes_resolved, y_nodes_resolved, use_node_scaler
         )
         
         kwargs_edges = self.convert_kwargs(X=X_edges_resolved, y=y_edges_resolved,
@@ -519,192 +507,185 @@ class DGLGraphMixin(MIXIN_BASE):
                                            reuse_if_existing=reuse_if_existing,
                                            *args, **kwargs)
         if featurize_edges:
+            if not isinstance(res._edges, pd.DataFrame):
+                if 'cudf' in str(getmodule(res._edges)):
+                    warnings.warn("cudf not supported for featurizing edges, converting to pandas")
+                    res = res.edges(res._edges.to_pandas())  # type: ignore
             res = res._featurize_edges_to_dgl(
                 res,
                 **kwargs_edges
-                #X_edges_resolved, y_edges_resolved, use_edge_scaler
             )
         if not inplace:
             return res
 
     def _mask_nodes(self):
-        if config.FEATURE in self.DGL_graph.ndata:
-            n = self.DGL_graph.ndata[config.FEATURE].shape[0]
+        if config.FEATURE in self._dgl_graph.ndata:
+            n = self._dgl_graph.ndata[config.FEATURE].shape[0]
             (
-                self.DGL_graph.ndata[config.TRAIN_MASK],
-                self.DGL_graph.ndata[config.TEST_MASK],
+                self._dgl_graph.ndata[config.TRAIN_MASK],
+                self._dgl_graph.ndata[config.TEST_MASK],
             ) = get_torch_train_test_mask(n, self.train_split)
 
     def _mask_edges(self):
-        if config.FEATURE in self.DGL_graph.edata:
-            n = self.DGL_graph.edata[config.FEATURE].shape[0]
+        if config.FEATURE in self._dgl_graph.edata:
+            n = self._dgl_graph.edata[config.FEATURE].shape[0]
             (
-                self.DGL_graph.edata[config.TRAIN_MASK],
-                self.DGL_graph.edata[config.TEST_MASK],
+                self._dgl_graph.edata[config.TRAIN_MASK],
+                self._dgl_graph.edata[config.TEST_MASK],
             ) = get_torch_train_test_mask(n, self.train_split)
 
-    def __getitem__(self, idx):
-        # get one example by index, here we have only one graph. #todo parameterize case if we have RGNN
-        if self.DGL_graph is None:
-            logger.warning("DGL graph is not built, run `g.build_gnn(...)` first")
-        return self.DGL_graph
-
-    # def __len__(self): # this messes up scope.
-    #     # number of data examples
-    #     return 1
 
 
 # if __name__ == "__main__":
-    # import graphistry
-    # from graphistry.networks import LinkPredModelMultiOutput
-    # from graphistry.ai_utils import setup_logger
-    # from data import get_botnet_dataframe
-    #
-    # import torch
-    # import torch.nn.functional as F
-    #
-    # logger = setup_logger("Main in DGL_utils", verbose=False)
-    #
-    # edf = get_botnet_dataframe(15000)
-    # edf = edf.drop_duplicates()
-    # src, dst = "to_node", "from_node"
-    # edf["to_node"] = edf.SrcAddr
-    # edf["from_node"] = edf.DstAddr
-    #
-    # good_cols_without_label = [
-    #     "Dur",
-    #     "Proto",
-    #     "Sport",
-    #     "Dport",
-    #     "State",
-    #     "TotPkts",
-    #     "TotBytes",
-    #     "SrcBytes",
-    #     "to_node",
-    #     "from_node",
-    # ]
-    #
-    # good_cols_without_label_or_edges = [
-    #     "Dur",
-    #     "Proto",
-    #     "Sport",
-    #     "Dport",
-    #     "State",
-    #     "TotPkts",
-    #     "TotBytes",
-    #     "SrcBytes",
-    # ]
-    #
-    # node_cols = ["Dur", "TotPkts", "TotBytes", "SrcBytes", "ip"]
-    #
-    # use_cols = ["Dur", "TotPkts", "TotBytes", "SrcBytes"]
-    #
-    # T = edf.Label.apply(
-    #     lambda x: 1 if "Botnet" in x else 0
-    # )  # simple indicator, useful for slicing later df.loc[T==1]
-    #
-    # y_edges = pd.DataFrame(
-    #     {"Label": edf.Label.values}, index=edf.index
-    # )  # must include index or g._MASK will through error
-    #
-    # # we can make an effective node_df using edf
-    # tdf = edf.groupby(["to_node"], as_index=False).mean().assign(ip=lambda x: x.to_node)
-    # fdf = (
-    #     edf.groupby(["from_node"], as_index=False)
-    #     .mean()
-    #     .assign(ip=lambda x: x.from_node)
-    # )
-    # ndf = pd.concat([tdf, fdf], axis=0)
-    # ndf = ndf.fillna(0)
-    #
-    # ndf = ndf[node_cols]
-    # ndf = ndf.drop_duplicates(subset=["ip"])
-    #
-    # src, dst = "from_node", "to_node"
-    # g = graphistry.edges(edf, src, dst).nodes(ndf, "ip")
-    #
-    # g2 = g.build_gnn(
-    #     "ip",
-    #     y_edges=y_edges,
-    #     use_edge_columns=good_cols_without_label,
-    #     use_node_columns=use_cols,
-    #     use_node_scaler="robust",
-    #     use_edge_scaler="robust",
-    # )
-    # # the DGL graph
-    # G = g2.DGL_graph
-    #
-    # # to get a sense of the different parts in training loop above
-    # # labels = torch.tensor(T.values, dtype=torch.float)
-    # train_mask = G.edata["train_mask"]
-    # test_mask = G.edata["test_mask"]
-    #
-    # # define the model
-    # n_feat = G.ndata["feature"].shape[1]
-    # latent_dim = 32
-    # n_output_feats = (
-    #     16  # this is equal to the latent dim output of the SAGE net, not n_targets
-    # )
-    #
-    # node_features = G.ndata["feature"].float()
-    # edge_label = G.edata["target"]
-    # n_targets = edge_label.shape[1]
-    # labels = edge_label.argmax(1)
-    # train_mask = G.edata["train_mask"]
-    #
-    # # instantiate model
-    # model = LinkPredModelMultiOutput(
-    #     n_feat, latent_dim, n_output_feats, n_targets
-    # )  # 1) #LinkPredModel(n_feat, latent_dim, n_output_feats)
-    #
-    # pred = model(G, node_features)  # the untrained graph
-    #
-    # print(
-    #     f"output of model should have same length as the number of edges: {pred.shape[0]}"
-    # )
-    # print(f"number of edges: {G.num_edges()}")
-    # assert G.num_edges() == pred.shape[0], "something went wrong"
-    #
-    # # the optimizer does all the backprop
-    # opt = torch.optim.Adam(model.parameters())
-    #
-    # def evaluate(model, graph, features, labels, mask):
-    #     model.eval()
-    #     with torch.no_grad():
-    #         logits = model(graph, features)
-    #         logits = logits[mask]
-    #         labels = labels[mask]
-    #         _, indices = torch.max(logits, dim=1)
-    #         correct = torch.sum(indices == labels.argmax(1))
-    #         return correct.item() * 1.0 / len(labels)
-    #
-    # use_cross_entropy_loss = True
-    # # train the model
-    # for epoch in range(2000):
-    #     logits = model(G, node_features)
-    #
-    #     if use_cross_entropy_loss:
-    #         loss = F.cross_entropy(logits[train_mask], edge_label[train_mask])
-    #     else:
-    #         loss = ((logits[train_mask] - edge_label[train_mask]) ** 2).mean()
-    #
-    #     pred = logits.argmax(1)
-    #     acc = sum(pred[test_mask] == labels[test_mask]) / len(pred[test_mask])
-    #
-    #     opt.zero_grad()
-    #     loss.backward()
-    #     opt.step()
-    #     if epoch % 100 == 0:
-    #         print(
-    #             f"epoch: {epoch} --------\nloss: {loss.item():.4f}\n\taccuracy: {acc:.4f}"
-    #         )
-    #
-    # # trained comparison
-    # logits = model(G, node_features)
-    # pred = logits.argmax(1)
-    #
-    # accuracy = sum(pred[test_mask] == labels[test_mask]) / len(
-    #     pred[test_mask]
-    # )  # does pretty well!
-    # print("-" * 60)
-    # print(f"Final Accuracy: {100 * accuracy:.2f}%")
+#     import graphistry
+#     from graphistry.networks import LinkPredModelMultiOutput
+#     #from graphistry.util import setup_logger
+    
+#     import torch
+#     import torch.nn.functional as F
+    
+#     #logger = setup_logger("Main in DGL_utils", verbose=True)
+    
+#     edf = edf = pd.read_csv('https://gist.githubusercontent.com/silkspace/33bde3e69ae24fee1298a66d1e00b467/raw/dc66bd6f1687270be7098f94b3929d6a055b4438/malware_bots.csv', index_col=0)
+#     edf = edf.drop_duplicates()
+#     edf = edf.sample(100000).reset_index(drop=True)
+#     src, dst = "to_node", "from_node"
+#     edf["to_node"] = edf.SrcAddr
+#     edf["from_node"] = edf.DstAddr
+    
+#     good_cols_without_label = [
+#         "Dur",
+#         "Proto",
+#         "Sport",
+#         "Dport",
+#         "State",
+#         "TotPkts",
+#         "TotBytes",
+#         "SrcBytes",
+#         "to_node",
+#         "from_node",
+#     ]
+    
+#     good_cols_without_label_or_edges = [
+#         "Dur",
+#         "Proto",
+#         "Sport",
+#         "Dport",
+#         "State",
+#         "TotPkts",
+#         "TotBytes",
+#         "SrcBytes",
+#     ]
+    
+#     node_cols = ["Dur", "TotPkts", "TotBytes", "SrcBytes", "ip"]
+    
+#     use_cols = ["Dur", "TotPkts", "TotBytes", "SrcBytes"]
+    
+#     T = edf.Label.apply(
+#         lambda x: 1 if "Botnet" in x else 0
+#     )  # simple indicator, useful for slicing later df.loc[T==1]
+    
+#     y_edges = pd.DataFrame(
+#         {"Label": edf.Label.values}, index=edf.index
+#     )  # must include index or g._MASK will through error
+    
+#     # we can make an effective node_df using edf
+#     tdf = edf.groupby(["to_node"], as_index=False).mean().assign(ip=lambda x: x.to_node)
+#     fdf = (
+#         edf.groupby(["from_node"], as_index=False)
+#         .mean()
+#         .assign(ip=lambda x: x.from_node)
+#     )
+#     ndf = pd.concat([tdf, fdf], axis=0)
+#     ndf = ndf.fillna(0)
+    
+#     ndf = ndf[node_cols]
+#     ndf = ndf.drop_duplicates(subset=["ip"])
+    
+#     src, dst = "from_node", "to_node"
+#     g = graphistry.edges(edf, src, dst).nodes(ndf, "ip")
+    
+#     g2 = g.build_gnn(
+#         y_edges=y_edges,
+#         X_edges=good_cols_without_label_or_edges,
+#         X_nodes=use_cols,
+#         use_node_scaler="zscale",
+#         use_edge_scaler="zscale",
+#     )
+#     # the DGL graph
+#     G = g2._dgl_graph
+#     print('G', G)
+#     # to get a sense of the different parts in training loop above
+#     # labels = torch.tensor(T.values, dtype=torch.float)
+#     train_mask = G.edata["train_mask"]
+#     test_mask = G.edata["test_mask"]
+    
+#     # define the model
+#     n_feat = G.ndata["feature"].shape[1]
+#     latent_dim = 32
+#     n_output_feats = (
+#         16  # this is equal to the latent dim output of the SAGE net, not n_targets
+#     )
+    
+#     node_features = G.ndata["feature"].float()
+#     edge_label = G.edata["target"]
+#     n_targets = edge_label.shape[1]
+#     labels = edge_label.argmax(1)
+#     train_mask = G.edata["train_mask"]
+    
+#     # instantiate model
+#     model = LinkPredModelMultiOutput(
+#         n_feat, latent_dim, n_output_feats, n_targets
+#     )  # 1) #LinkPredModel(n_feat, latent_dim, n_output_feats)
+    
+#     pred = model(G, node_features)  # the untrained graph
+    
+#     print(
+#         f"output of model should have same length as the number of edges: {pred.shape[0]}"
+#     )
+#     print(f"number of edges: {G.num_edges()}")
+#     assert G.num_edges() == pred.shape[0], "something went wrong"
+    
+#     # the optimizer does all the backprop
+#     opt = torch.optim.Adam(model.parameters())
+    
+#     def evaluate(model, graph, features, labels, mask):
+#         model.eval()
+#         with torch.no_grad():
+#             logits = model(graph, features)
+#             logits = logits[mask]
+#             labels = labels[mask]
+#             _, indices = torch.max(logits, dim=1)
+#             correct = torch.sum(indices == labels.argmax(1))
+#             return correct.item() * 1.0 / len(labels)
+    
+#     use_cross_entropy_loss = False
+#     # train the model
+#     for epoch in range(2000):
+#         logits = model(G, node_features)
+    
+#         if use_cross_entropy_loss:
+#             loss = F.cross_entropy(logits[train_mask], edge_label[train_mask])
+#         else:
+#             loss = ((logits[train_mask] - edge_label[train_mask]) ** 2).mean()
+    
+#         pred = logits.argmax(1)
+#         acc = sum(pred[test_mask] == labels[test_mask]) / len(pred[test_mask])
+    
+#         opt.zero_grad()
+#         loss.backward()
+#         opt.step()
+#         if epoch % 100 == 0:
+#             print(
+#                 f"epoch: {epoch} --------\nloss: {loss.item():.4f}\n\taccuracy: {acc:.4f}"
+#             )
+    
+#     # trained comparison
+#     logits = model(G, node_features)
+#     pred = logits.argmax(1)
+    
+#     accuracy = sum(pred[test_mask] == labels[test_mask]) / len(
+#         pred[test_mask]
+#     )  # does pretty well, more data gets it up to 90+%, like RFRegressor!
+#     print("-" * 60)
+#     print(f"Final Accuracy: {100 * accuracy:.2f}%")
