@@ -1,4 +1,4 @@
-from typing import Any, Callable, Iterable, List, Optional, Set, Union, TYPE_CHECKING
+from typing import Any, Callable, Iterable, List, Optional, Set, Union, TYPE_CHECKING, cast
 import os, pandas as pd
 from .Plottable import Plottable
 try:
@@ -18,13 +18,6 @@ except:
 from .util import setup_logger
 logger = setup_logger(__name__)
 
-
-if TYPE_CHECKING:
-    MIXIN_BASE = Plottable
-else:
-    MIXIN_BASE = object
-
-
 def clean_str(v):
     if isinstance(v, str):
         return v.replace('"', r'\"')
@@ -381,17 +374,21 @@ def resultset_to_g_structured_item(
 
 DROP_QUERY = 'g.V().drop()'
 
-class GremlinMixin(MIXIN_BASE):
+class GremlinMixin(Plottable):
     """
     Universal Gremlin<>pandas/graphistry functionality across Gremlin connectors
     
     Currently serializes queries as strings instead of bytecode in order to support cosmosdb
     """
 
-    _reconnect_gremlin : Optional[Callable[['GremlinMixin'], 'GremlinMixin']] = None
+    _reconnect_gremlin : Optional[Callable[[Plottable], Plottable]] = None
     _gremlin_client : Optional[Client]
 
     def __init__(self, *args, gremlin_client: Optional[Client] = None, **kwargs):
+        # NOTE: Cooperative constructor passes along all args/kwargs for other mixins
+        kwargs['gremlin_client'] = gremlin_client
+        super().__init__(*args, **kwargs)
+
         if gremlin_client is not None:
             self._gremlin_client = gremlin_client
 
@@ -399,33 +396,45 @@ class GremlinMixin(MIXIN_BASE):
         self,
         gremlin_client: Client
     ):
-        """Pass in a generic gremlin python client
+        """
+        Set the Gremlin client to interact with the Gremlin-enabled database.
 
-            **Example: Login and plot **
+        This method allows you to pass a custom Gremlin Python client to interact with databases like CosmosDB 
+        using Gremlin queries. It stores the client for subsequent queries in the Graphistry workflow.
+
+        :param gremlin_client: Instance of the Gremlin Python client.
+        :type gremlin_client: gremlin_python.driver.client.Client
+
+        :return: The instance of Graphistry, updated with the Gremlin client.
+        :rtype: GremlinMixin
+
+        **Example: Login and plot**
             ::
 
                 import graphistry
                 from gremlin_python.driver.client import Client
 
                 my_gremlin_client = Client(
-                f'wss://MY_ACCOUNT.gremlin.cosmosdb.azure.com:443/',
-                'g', 
-                username=f"/dbs/MY_DB/colls/{self.COSMOS_CONTAINER}",
-                password=self.COSMOS_PRIMARY_KEY,
-                message_serializer=GraphSONSerializersV2d0())
+                    f'wss://MY_ACCOUNT.gremlin.cosmosdb.azure.com:443/',
+                    'g', 
+                    username=f"/dbs/MY_DB/colls/{self.COSMOS_CONTAINER}",
+                    password=self.COSMOS_PRIMARY_KEY,
+                    message_serializer=GraphSONSerializersV2d0())
 
-                (graphistry
+                g = (graphistry
                     .gremlin_client(my_gremlin_client)
                     .gremlin('g.E().sample(10)')
                     .fetch_nodes()  # Fetch properties for nodes
-                    .plot())
+                )
+
+                g.plot()
 
         """    
 
         self._gremlin_client = gremlin_client
         return self
 
-    def connect(self):
+    def connect(self) -> Plottable:
         """
         Use previously provided credentials to connect. Disconnect any preexisting clients.
         """
@@ -473,19 +482,59 @@ class GremlinMixin(MIXIN_BASE):
        
     def gremlin(self, queries: Union[str, Iterable[str]]) -> Plottable:
         """
-            Run one or more gremlin queries and get back the result as a graph object
-            To support cosmosdb, sends as strings
+        Run one or more Gremlin queries and return the result as a PyGraphistry graph object.
 
-            **Example: Login and plot **
+        This method allows you to execute Gremlin queries, either as a single string or an iterable of strings, 
+        and retrieve the results in a format that PyGraphistry can process and visualize. 
+        To support databases like CosmosDB, the queries are sent as strings.
+
+        :param queries: One or more Gremlin queries to execute. Can be a single query (as a string) or multiple queries (as a list of strings).
+        :type queries: Union[str, Iterable[str]]
+
+        :return: A PyGraphistry `Plottable` graph object containing the query results.
+        :rtype: Plottable
+
+        **Example: Execute a Gremlin Query and Plot**
             ::
 
                 import graphistry
-                (graphistry
-                    .gremlin_client(my_gremlin_client)
-                    .gremlin('g.E().sample(10)')
-                    .fetch_nodes()  # Fetch properties for nodes
-                    .plot())
+                from gremlin_python.driver.client import Client
+                from gremlin_python.driver.serializer import GraphSONSerializersV2d0
 
+                # Create a Gremlin client for CosmosDB
+                my_gremlin_client = Client(
+                    f'wss://MY_ACCOUNT.gremlin.cosmosdb.azure.com:443/',
+                    'g',
+                    username=f"/dbs/MY_DB/colls/{self.COSMOS_CONTAINER}",
+                    password='MY_COSMOS_PRIMARY_KEY',
+                    message_serializer=GraphSONSerializersV2d0()
+                )
+
+                # Run a Gremlin query and visualize the result
+                graphistry \
+                    .gremlin_client(my_gremlin_client) \
+                    .gremlin('g.V().hasLabel("person").limit(5)') \
+                    .fetch_nodes() \
+                    .plot()
+
+            In this example, the Gremlin query selects the first 5 vertices with the label "person" and 
+            plots the result in PyGraphistry.
+
+        **Example: Running Multiple Gremlin Queries**
+            ::
+
+                queries = [
+                    'g.V().hasLabel("person").limit(5)',
+                    'g.E().limit(10)'
+                ]
+
+                graphistry \
+                    .gremlin_client(my_gremlin_client) \
+                    .gremlin(queries) \
+                    .fetch_nodes() \
+                    .plot()
+
+            This example demonstrates how to run multiple Gremlin queries, fetch their results, and visualize them.
         """
         ensure_imports()
         if isinstance(queries, str):
@@ -616,7 +665,7 @@ class GremlinMixin(MIXIN_BASE):
         node_id = g._node
         if node_id is None:
             node_id = 'id'
-            g = g.bind(node=node_id)
+            g = cast('GremlinMixin', g.bind(node=node_id))
         if nodes_df is None:
             edges_df = g._edges
             if g._edges is None:
@@ -687,18 +736,11 @@ class GremlinMixin(MIXIN_BASE):
         return g2
 
 
-if TYPE_CHECKING:
-    COSMOS_BASE = GremlinMixin
-    NEPTUNE_BASE = GremlinMixin
-else:
-    COSMOS_BASE = object
-    NEPTUNE_BASE = object
 
+class NeptuneMixin(GremlinMixin):
 
-class NeptuneMixin(NEPTUNE_BASE):
-
-    def __init__(self, *args, **kwargs):
-        pass
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
 
     def neptune(
         self,
@@ -812,16 +854,16 @@ class NeptuneMixin(NEPTUNE_BASE):
 
             return self
 
-        self._reconnect_gremlin : Optional[Callable[[NeptuneMixin], NeptuneMixin]] = connect  # type: ignore
+        self._reconnect_gremlin = connect  # type: ignore
 
         return self
 
 
 
-class CosmosMixin(COSMOS_BASE):
+class CosmosMixin(GremlinMixin):
 
-    def __init__(self, *args, **kwargs):
-        pass
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
 
     def cosmos(
         self,
@@ -836,7 +878,7 @@ class CosmosMixin(COSMOS_BASE):
            Environment variable names are the same as the constructor argument names
            If no client provided, create (connect)
 
-        **Example: Login and plot **
+        **Example: Login and plot**
                 ::
 
                     import graphistry
@@ -873,7 +915,7 @@ class CosmosMixin(COSMOS_BASE):
                 message_serializer=GraphSONSerializersV2d0())
             return self
 
-        self._reconnect_gremlin : Optional[Callable[[CosmosMixin], CosmosMixin]] = connect  # type: ignore
+        self._reconnect_gremlin = connect  # type: ignore
 
         if gremlin_client is None:
             if self._reconnect_gremlin is None:
