@@ -1,14 +1,15 @@
-import pandas as pd
 from typing import Any, Dict, List, Optional, Union
+import pandas as pd
+import warnings
+
 from graphistry.constants import NODE
-from graphistry.Engine import Engine
+from graphistry.Engine import EngineAbstract
 from graphistry.Plottable import Plottable
-from graphistry.plugins_types import CuGraphKind
+from graphistry.plugins_types.cugraph_types import CuGraphKind
 from graphistry.util import setup_logger
 logger = setup_logger(__name__)
 
 
-
 #import logging
 #logger.setLevel(logging.DEBUG)
 
@@ -38,7 +39,7 @@ def from_cugraph(self,
 ) -> Plottable:
     """
     
-    If bound IDs, use the same IDs in the returned graph.
+    Take input cugraph.Graph object and load in data and bindings (source, destination, edge_weight)
 
     If non-empty nodes/edges, instead of returning G's topology, use existing topology and merge in G's attributes
     
@@ -50,8 +51,28 @@ def from_cugraph(self,
 
     ####
 
-    src = self._source or SRC_CUGRAPH
-    dst = self._destination or DST_CUGRAPH
+    if hasattr(G, 'source_columns') and G.source_columns is not None:
+        s = G.source_columns
+        if isinstance(s, list):
+            s = s[0]
+        assert isinstance(s, str), "Found G.source_columns, and expected it to be a string or a list of one string, but was: %s" % G.souurce_columns
+        if self._source is not None and self._source != s:
+            warnings.warn('Switching g source column name to G source column name')
+    else:
+        s = self._source or SRC_CUGRAPH
+    src = s
+
+    if hasattr(G, 'destination_columns') and G.destination_columns is not None:
+        d = G.destination_columns
+        if isinstance(d, list):
+            d = d[0]
+        assert isinstance(d, str), "Found G.destination_columns, and expected it to be a string or a list of one string, but was: %s" % G.destination_columns
+        if self._destination is not None and self._destination != d:
+            warnings.warn('Switching g destination column name to G destination column name')
+    else:
+        d = self._destination or DST_CUGRAPH
+    dst = d
+
     edges_gdf = G.view_edge_list()  # src, dst
 
     if g._nodes is not None and load_nodes:
@@ -175,7 +196,6 @@ node_compute_algs_to_attr : Dict[str, Union[str, List[str]]] = {
     'spectralModularityMaximizationClustering': 'cluster',
     'connected_components': 'labels',
     'strongly_connected_components': 'labels',
-    'weakly_connected_components': 'labels',
     'core_number': 'core_number',
     #'k_core': 'values',
     'hits': ['hubs', 'authorities'],
@@ -189,7 +209,7 @@ node_compute_algs_to_attr : Dict[str, Union[str, List[str]]] = {
 }
 
 #'src'/'dst' or 'source'/'destination' +
-edge_compute_algs_to_attr = {
+edge_compute_algs_to_attr: Dict[str, str] = {
     'batched_ego_graphs': 'unknown',
     'edge_betweenness_centrality': 'edge_betweenness_centrality',
     'jaccard': 'jaccard_coeff',
@@ -211,9 +231,9 @@ graph_compute_algs = [
     'minimum_spanning_tree'
 ]
 
-compute_algs = list(node_compute_algs_to_attr.keys()) + list(edge_compute_algs_to_attr.keys()) + graph_compute_algs
+compute_algs: List[str] = list(node_compute_algs_to_attr.keys()) + list(edge_compute_algs_to_attr.keys()) + graph_compute_algs
 
-def compute_cugraph(
+def compute_cugraph_core(
     self: Plottable,
     alg: str, out_col: Optional[str] = None, params: dict = {},
     kind : CuGraphKind = 'Graph', directed = True,
@@ -223,35 +243,56 @@ def compute_cugraph(
 
     :param alg: algorithm name
     :type alg: str
+
     :param out_col: node table output column name, defaults to alg param
     :type out_col: Optional[str]
+
     :param params: algorithm parameters passed to cuGraph as kwargs
     :type params: dict
+
     :param kind: kind of cugraph to use
     :type kind: CuGraphKind
+
     :param directed: whether graph is directed
     :type directed: bool
+
     :param G: cugraph graph to use; if None, use self
     :type G: Optional[cugraph.Graph]
 
     :return: Plottable
     :rtype: Plottable
-    
+
+    **Example: Pass params to cugraph**
+        ::
+
+            edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
+            g = graphistry.edges(edges, 's', 'd')
+            g2 = g.compute_cugraph('betweenness_centrality', params={'k': 2})
+            assert 'betweenness_centrality' in g2._nodes.columns
+
     **Example: Pagerank**
         ::
+
+            edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
+            g = graphistry.edges(edges, 's', 'd')
             g2 = g.compute_cugraph('pagerank')
             assert 'pagerank' in g2._nodes.columns
 
+    **Example: Personalized Pagerank**
+        ::
+            edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
+            g = graphistry.edges(edges, 's', 'd')
+            g2 = g.compute_cugraph('pagerank', params={'personalization': cudf.DataFrame({'vertex': ['a'], 'values': [1]})})
+            assert 'pagerank' in g2._nodes.columns
+
     **Example: Katz centrality with rename**
         ::
+
+            edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
+            g = graphistry.edges(edges, 's', 'd')
             g2 = g.compute_cugraph('katz_centrality', out_col='katz_centrality_renamed')
             assert 'katz_centrality_renamed' in g2._nodes.columns
 
-    **Example: Pass params to cugraph**
-        ::
-            g2 = g.compute_cugraph('k_truss', params={'k': 2})
-            assert 'k_truss' in g2._nodes.columns
-
     """
 
     import cugraph
@@ -263,7 +304,7 @@ def compute_cugraph(
         out = getattr(cugraph, alg)(G, **params)
         if isinstance(out, tuple):
             out = out[0]
-        g = self.materialize_nodes(engine=Engine.CUDF)
+        g = self.materialize_nodes(engine=EngineAbstract.CUDF)
         if g._node != 'vertex':
             out = out.rename(columns={'vertex': g._node})
         expected_cols = node_compute_algs_to_attr[alg]
@@ -306,17 +347,88 @@ def compute_cugraph(
             out = out[0]
         if out_col is not None:
             raise ValueError('Graph returned, but out_col was specified')
-        return from_cugraph(self, out, load_nodes=False)
+        self2 = self
+        if self._source != out.source_columns:
+            logger.debug('Switching g source column name to G source column name to work around cugraph inconsistency')
+            if out.source_columns == 'src':
+                self2 = self.edges(self._edges.rename(columns={self._source: 'src', self._destination: 'dst'}), 'src', 'dst')
+        res = from_cugraph(self2, out, load_nodes=False)
+        if not (self2 is self):
+            res = res.edges(self._edges, self._source, self._destination)
+        return res
 
     raise ValueError('Unsupported algorithm: %s', alg)
 
 
+def _is_retryable_cugraph_call(self: Plottable) -> bool:
+    """
+    Whether can retry with Plottable cudf numeric graph indexes coerced to strings
+    """
+    if self._edges is not None:
+        import cudf
+        from cudf.api.types import is_numeric_dtype
+        return (
+            isinstance(self._edges, cudf.DataFrame)
+            and (self._source is not None)
+            and (self._source in self._edges.columns)
+            and is_numeric_dtype(self._edges[self._source])
+        )
+    return False
 
-layout_algs = [
+def _coerce_and_retry_cugraph(self: Plottable, core_fn, *args, **kwargs):
+    """
+    Attempts to run the core function and retries with coerced edge/node types if the first attempt fails
+    """
+
+    assert _is_retryable_cugraph_call(self), "Check _is_retryable_cugraph_call before calling this function"
+
+    logger.warning('Failed to run cugraph algorithm and src/dst columns are numeric, coercing to strings and retrying')
+    g2 = self.edges(self._edges.assign(
+        **{self._source: self._edges[self._source].astype(str),
+            self._destination: self._edges[self._destination].astype(str)}))
+    if g2._nodes is not None and g2._node is not None and g2._node in g2._nodes.columns:
+        g2 = g2.nodes(g2._nodes.assign(
+            **{self._node: g2._nodes[g2._node].astype(str)}))
+    g_computed = core_fn(g2, *args, **kwargs)
+    # revert to original dtype
+    dtype = self._edges[self._source].dtype
+    g_out = g2.edges(g_computed._edges.assign(
+        **{g_computed._source: g_computed._edges[g_computed._source].astype(dtype),
+            g_computed._destination: g_computed._edges[g_computed._destination].astype(dtype)}
+    ))
+    if g_computed._nodes is not None:
+        g_out = g_out.nodes(g_computed._nodes.assign(
+            **{g_computed._node: g_computed._nodes[g_out._node].astype(dtype)}
+        ))
+    return g_out
+
+
+def compute_cugraph(
+    self: Plottable,
+    alg: str, out_col: Optional[str] = None, params: dict = {},
+    kind : CuGraphKind = 'Graph', directed = True,
+    G: Optional[Any] = None
+) -> Plottable:
+    try:
+        return compute_cugraph_core(self, alg, out_col, params, kind, directed, G)
+    except ValueError as e:
+        if _is_retryable_cugraph_call(self):
+            try:
+                return _coerce_and_retry_cugraph(self, compute_cugraph_core, alg, out_col, params, kind, directed, G)
+            except ValueError as e2:
+                logger.error('Failed to run cugraph algorithm even with numeric->string coercion: %s', e2, exc_info=True)
+                raise e
+        raise e
+
+
+compute_cugraph.__doc__ = compute_cugraph_core.__doc__
+
+
+layout_algs: List[str] = [
     'force_atlas2'
 ]
 
-def layout_cugraph(
+def layout_cugraph_core(
     self: Plottable,
     layout: str = 'force_atlas2', params: dict = {},
     kind : CuGraphKind = 'Graph', directed = True,
@@ -360,6 +472,7 @@ def layout_cugraph(
 
     **Example: ForceAtlas2 layout**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
@@ -367,6 +480,7 @@ def layout_cugraph(
 
     **Example: Change which column names are generated**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
@@ -377,6 +491,7 @@ def layout_cugraph(
 
     **Example: Pass parameters to layout methods**
         ::
+        
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
@@ -386,7 +501,7 @@ def layout_cugraph(
    
     import cugraph
 
-    g = self.materialize_nodes(engine=Engine.CUDF)
+    g = self.materialize_nodes(engine=EngineAbstract.CUDF)
 
     if layout not in layout_algs:
         raise ValueError('Unsupported algorithm: %s', layout)
@@ -412,3 +527,28 @@ def layout_cugraph(
     if play is not None:
         g2 = g2.layout_settings(play=play)
     return g2
+
+
+def layout_cugraph(
+    self: Plottable,
+    layout: str = 'force_atlas2', params: dict = {},
+    kind : CuGraphKind = 'Graph', directed = True,
+    G: Optional[Any] = None,
+    bind_position: bool = True,
+    x_out_col: str = 'x',
+    y_out_col: str = 'y',
+    play: Optional[int] = 0,
+) -> Plottable:
+    try:
+        return layout_cugraph_core(self, layout, params, kind, directed, G, bind_position, x_out_col, y_out_col, play)
+    except ValueError as e:
+        if _is_retryable_cugraph_call(self):
+            try:
+                return _coerce_and_retry_cugraph(self, layout_cugraph_core, layout, params, kind, directed, G, bind_position, x_out_col, y_out_col, play)
+            except ValueError as e2:
+                logger.error('Failed to run cugraph layout even with numeric->string coercion: %s', e2, exc_info=True)
+                raise e
+        raise e
+
+
+layout_cugraph.__doc__ = layout_cugraph_core.__doc__
