@@ -1,5 +1,5 @@
 import pandas as pd
-from typing import Any, List, Optional
+from typing import List, Optional, Any
 from graphistry.constants import NODE
 from graphistry.Plottable import Plottable
 from graphistry.util import setup_logger
@@ -15,7 +15,7 @@ DST_IGRAPH = 'target'
 NODE_IGRAPH = NODE
 
 def from_igraph(self,
-    ig,
+    ig: Any,
     node_attributes: Optional[List[str]] = None,
     edge_attributes: Optional[List[str]] = None,
     load_nodes: bool = True, load_edges: bool = True,
@@ -49,10 +49,10 @@ def from_igraph(self,
     :param merge_if_existing: bool
 
     :returns: Plotter
-    :rtype: Plotter
 
     **Example: Convert from igraph, including all node/edge properties**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a', 'b', 'c', 'd'], 'd': ['b', 'c', 'd', 'e'], 'v': [101, 102, 103, 104]})
             g = graphistry.edges(edges, 's', 'd').materialize_nodes().get_degrees()
@@ -62,6 +62,7 @@ def from_igraph(self,
 
     **Example: Enrich from igraph, but only load in 1 node attribute**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a', 'b', 'c', 'd'], 'd': ['b', 'c', 'd', 'e'], 'v': [101, 102, 103, 104]})
             g = graphistry.edges(edges, 's', 'd').materialize_nodes().get_degree()
@@ -104,6 +105,8 @@ def from_igraph(self,
             nodes_df = nodes_df[ node_attributes ]
 
         if g._nodes is not None and merge_if_existing:
+            if g._node is None:
+                raise ValueError('Non-None g._nodes and merge_if_existing=True, yet no g._node is defined')
             if len(g._nodes) != len(nodes_df):
                 logger.warning('node tables do not match in length; switch merge_if_existing to False or load_nodes to False or add missing nodes')
 
@@ -125,8 +128,21 @@ def from_igraph(self,
             node_id_col = None
         elif node_col in ig_vs_df:
             node_id_col = node_col
+        #User to_igraph() with numeric IDs may swizzle id mappings (ex: sparse numeric) so try to un-swizzle
+        #FIXME: how to handle dense edge case's swizzling?
         elif g._node is not None and g._nodes[g._node].dtype.name == ig_vs_df.reset_index()['vertex ID'].dtype.name:
-            node_id_col = None
+            found = False
+            #FIXME: This seems quite error prone... what if any fields already exist?
+            for c in ['name', 'id', 'idx', NODE]:
+                if c in ig_vs_df.columns:
+                    if g._nodes[g._node].min() == ig_vs_df[c].min() and g._nodes[g._node].max() == ig_vs_df[c].max():
+                        if g._nodes[g._node].sort_values().equals(ig_vs_df[c].sort_values()):
+                            node_id_col = c
+                            found = True
+                            break
+            if not found:
+                logger.debug('lacks matching sortable dimension, likely passed integers-as-vids, continue without remapping')
+                node_id_col = None
         elif 'name' in ig_vs_df:
             node_id_col = 'name'
         else:
@@ -198,13 +214,14 @@ def from_igraph(self,
     return g
 
 
-def to_igraph(self: Plottable, 
+def to_igraph(
+    self: Plottable, 
     directed: bool = True,
     include_nodes: bool = True,
     node_attributes: Optional[List[str]] = None,
     edge_attributes: Optional[List[str]] = None,
     use_vids: bool = False
-):
+) -> Any:
     """Convert current item to igraph Graph . See examples in from_igraph.
 
     :param directed: Whether to create a directed graph (default True)
@@ -247,7 +264,8 @@ def to_igraph(self: Plottable,
     )
 
 
-compute_algs = [
+compute_algs: List[str] = [
+    'articulation_points',
     'authority_score',
     'betweenness',
     'bibcoupling',
@@ -265,6 +283,7 @@ compute_algs = [
     'community_leading_eigenvector',
     'community_leiden',
     'community_multilevel',
+    'community_optimal_modularity',
     'community_spinglass',
     'community_walktrap',
     'constraint',
@@ -277,6 +296,7 @@ compute_algs = [
     'k_core',
     #'modularity',
     'pagerank',
+    'personalized_pagerank',
     'spanning_tree'
 ]
 
@@ -286,7 +306,8 @@ def compute_igraph(
     out_col: Optional[str] = None,
     directed: Optional[bool] = None,
     use_vids=False,
-    params: dict = {}
+    params: dict = {},
+    stringify_rich_types=True
 ) -> Plottable:
     """Enrich or replace graph using igraph methods
 
@@ -305,42 +326,57 @@ def compute_igraph(
     :param params: Any named parameters to pass to the underlying igraph method
     :type params: dict
 
+    :param stringify_rich_types: When rich types like igraph.Graph are returned, which may be problematic for downstream rendering, coerce them to strings
+    :type stringify_rich_types: bool 
+
     :returns: Plotter
     :rtype: Plotter
 
     **Example: Pagerank**
-
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
-            g = graphistry.edges(edges, 's', 'd')
+            g = graphistry.edges(edges, 's', 'd').materialize_nodes()
             g2 = g.compute_igraph('pagerank')
             assert 'pagerank' in g2._nodes.columns
 
     **Example: Pagerank with custom name**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
-            g = graphistry.edges(edges, 's', 'd')
+            g = graphistry.edges(edges, 's', 'd').materialize_nodes()
             g2 = g.compute_igraph('pagerank', out_col='my_pr')
             assert 'my_pr' in g2._nodes.columns
 
     **Example: Pagerank on an undirected**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
-            g = graphistry.edges(edges, 's', 'd')
+            g = graphistry.edges(edges, 's', 'd').materialize_nodes()
             g2 = g.compute_igraph('pagerank', directed=False)
             assert 'pagerank' in g2._nodes.columns
 
     **Example: Pagerank with custom parameters**
-            ::
+        ::
+
                 import graphistry, pandas as pd
                 edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
-                g = graphistry.edges(edges, 's', 'd')
+                g = graphistry.edges(edges, 's', 'd').materialize_nodes()
                 g2 = g.compute_igraph('pagerank', params={'damping': 0.85})
                 assert 'pagerank' in g2._nodes.columns
 
+    **Example: Personalized Pagerank**
+        ::
+
+            import graphistry, pandas as pd
+            edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['c','c','e','e']})
+            g = graphistry.edges(edges, 's', 'd').materialize_nodes()
+            g2 = g.compute_igraph('personalized_pagerank')
+            assert 'personalized_pagerank' in g2._nodes.columns
+
     """
 
     import igraph
@@ -369,17 +405,32 @@ def compute_igraph(
         return from_igraph(self, out)
     elif isinstance(out, list) and self._nodes is None:
         raise ValueError("No g._nodes table found; use .bind(), .nodes(), .materialize_nodes()")
-    elif len(out) == len(self._nodes):
-        clustering = out
+    elif alg == 'articulation_points':
+        assert isinstance(out, list)  # List[int]
+        membership = [0] * len(ig.vs)
+        for i in out:
+            membership[i] = 1
+        clustering = membership
+    elif isinstance(out, list) and len(out) == len(self._nodes):
+        if stringify_rich_types and len(out) > 0 and all((isinstance(c, igraph.Graph) for c in out)):
+            #ex: k_core
+            clustering = [str(c) for c in out]
+        else:
+            clustering = out
     else:
-        raise RuntimeError(f'Unexpected output type "{type(out)}"; should be VertexClustering, VertexDendrogram, Graph, or list_<|V|>')    
+        if isinstance(out, list) and len(out) > 0:
+            xtra = f" (element 0 type: {type(out[0])})"
+        else:
+            xtra = ""
+
+        raise RuntimeError(f'Unexpected output type "{type(out)}"{xtra}; should be VertexClustering, VertexDendrogram, Graph, or list_<|V|>')
 
     ig.vs[out_col] = clustering
 
     return self.from_igraph(ig)
 
 
-layout_algs = [
+layout_algs: List[str] = [
     'auto', 'automatic',
     'bipartite',
     'circle', 'circular',
@@ -447,6 +498,7 @@ def layout_igraph(
 
     **Example: Sugiyama layout**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
@@ -456,6 +508,7 @@ def layout_igraph(
 
     **Example: Change which column names are generated**
         ::
+
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
@@ -466,6 +519,7 @@ def layout_igraph(
 
     **Example: Pass parameters to layout methods - Sort nodes by degree**
         ::
+        
             import graphistry, pandas as pd
             edges = pd.DataFrame({'s': ['a','b','c','d'], 'd': ['b','c','d','e']})
             g = graphistry.edges(edges, 's', 'd')
