@@ -1,3 +1,4 @@
+from typing import Any
 import hashlib
 import logging
 import os
@@ -9,23 +10,45 @@ import string
 import uuid
 import warnings
 from functools import lru_cache
-from typing import Any
 
+from graphistry.models.ModelDict import ModelDict
 from .constants import VERBOSE, CACHE_COERCION_SIZE, TRACE
 
 
 # #####################################
-def setup_logger(name, verbose=VERBOSE, fullpath=TRACE):
-    if fullpath:
-        FORMAT = "[%(filename)s:%(lineno)s - %(funcName)20s() ]\n   %(message)s\n"
+
+@lru_cache(maxsize=1)
+def get_handler(short=False):
+    if short:
+        formatter = logging.Formatter("%(filename)s:%(lineno)s %(message)s\n")
     else:
-        FORMAT = " %(message)s\n"
-    logging.basicConfig(format=FORMAT)
-    logger = logging.getLogger(f'graphistry.{name}')
-    if verbose is None:
-        logger.setLevel(logging.ERROR)
-    else:
-        logger.setLevel(logging.INFO if verbose else logging.DEBUG)
+        formatter = logging.Formatter("\n[%(filename)s:%(lineno)s - %(funcName)20s() ]   %(message)s\n")
+    handler = logging.StreamHandler()
+    handler.setFormatter(formatter)
+    return handler
+
+def setup_logger(name='', verbose=VERBOSE, fullpath=TRACE):
+    logger = logging.getLogger(name)
+
+    if verbose is not None:
+        if verbose:
+            logger.setLevel(logging.DEBUG)
+        else:
+            logger.setLevel(logging.ERROR)
+    elif os.environ.get('LOG_LEVEL', None) is not None:
+        if os.environ['LOG_LEVEL'] == 'TRACE':
+            logger.setLevel(logging.DEBUG)
+        else:
+            logger.setLevel(os.environ['LOG_LEVEL'])
+
+    if (
+        not bool(logging.getLogger().handlers)
+        and not logger.handlers  # don't double-attach
+        and (verbose is not None or os.environ.get('LOG_LEVEL'))
+    ):
+        logger.addHandler(get_handler(short=False))
+
+    logger.propagate = True
     return logger
 
 
@@ -33,6 +56,8 @@ def setup_logger(name, verbose=VERBOSE, fullpath=TRACE):
 # Caching utils
 
 _cache_coercion_val = None
+
+
 @lru_cache(maxsize=CACHE_COERCION_SIZE)
 def cache_coercion_helper(k):
     return _cache_coercion_val
@@ -40,8 +65,8 @@ def cache_coercion_helper(k):
 
 def cache_coercion(k, v):
     """
-        Holds references to last 100 used coercions
-        Use with weak key/value dictionaries for actual lookups
+    Holds references to last 100 used coercions
+    Use with weak key/value dictionaries for actual lookups
     """
     global _cache_coercion_val
     _cache_coercion_val = v
@@ -59,30 +84,37 @@ class WeakValueWrapper:
 def hash_pdf(df: pd.DataFrame) -> str:
     # can be 20% faster via to_parquet (see lmeyerov issue in pandas gh), but unclear if always available
     return (
-        hashlib.sha256(putil.hash_pandas_object(df, index=True).to_numpy().tobytes()).hexdigest()
-        + hashlib.sha256(str(df.columns).encode('utf-8')).hexdigest()  # noqa: W503
+        hashlib.sha256(
+            putil.hash_pandas_object(df, index=True).to_numpy().tobytes()
+        ).hexdigest()
+        + hashlib.sha256(str(df.columns).encode("utf-8")).hexdigest()  # noqa: W503
     )
 
 
 def hash_memoize_helper(v: Any) -> str:
 
     if isinstance(v, dict):
-        rolling = '{'
+        rolling = "{"
         for k2, v2 in v.items():
-            rolling += f'{k2}:{hash_memoize_helper(v2)},'
-        rolling += '}'
+            rolling += f"{k2}:{hash_memoize_helper(v2)},"
+        rolling += "}"
+    elif isinstance(v, ModelDict):
+        rolling = "{"
+        for k2, v2 in v.items():
+            rolling += f"{k2}:{hash_memoize_helper(v2)},"
+        rolling += "}"
     elif isinstance(v, list):
-        rolling = '['
+        rolling = "["
         for i in v:
-            rolling += f'{hash_memoize_helper(i)},'
-        rolling += ']'
+            rolling += f"{hash_memoize_helper(i)},"
+        rolling += "]"
     elif isinstance(v, tuple):
-        rolling = '('
+        rolling = "("
         for i in v:
-            rolling += f'{hash_memoize_helper(i)},'
-        rolling += ')'
+            rolling += f"{hash_memoize_helper(i)},"
+        rolling += ")"
     elif isinstance(v, bool):
-        rolling = 'T' if v else 'F'
+        rolling = "T" if v else "F"
     elif isinstance(v, int):
         rolling = str(v)
     elif isinstance(v, float):
@@ -90,54 +122,20 @@ def hash_memoize_helper(v: Any) -> str:
     elif isinstance(v, str):
         rolling = v
     elif v is None:
-        rolling = 'N'
+        rolling = "N"
     elif isinstance(v, pd.DataFrame):
         rolling = hash_pdf(v)
     else:
-        raise TypeError(f'Unsupported memoization type: {type(v)}')
+        raise TypeError(f"Unsupported memoization type: {type(v)}")
 
     return rolling
 
+
 def hash_memoize(v: Any) -> str:
-    return hashlib.sha256(hash_memoize_helper(v).encode('utf-8')).hexdigest()
+    return hashlib.sha256(hash_memoize_helper(v).encode("utf-8")).hexdigest()
+
 
-def check_set_memoize(g, metadata, attribute, name: str = '', memoize: bool = True):  # noqa: C901
-    """
-        Helper Memoize function that checks if metadata args have changed for object g -- which is unconstrained save
-        for the fact that it must have `attribute`. If they have not changed, will return memoized version,
-        if False, will continue with whatever pipeline it is in front.
-    """
-    
-    logger = setup_logger(f'{__name__}.memoization')
 
-    if not memoize:
-        logger.debug('Memoization disabled')
-        return False
-    
-    hashed = None
-    weakref = getattr(g, attribute)
-    try:
-        hashed = hash_memoize(metadata)
-    except TypeError:
-        logger.warning(
-            f'! Failed {name} speedup attempt. Continuing without memoization speedups.'
-        )
-    try:
-        if hashed in weakref:
-            logger.debug(f'{name} memoization hit: %s', hashed)
-            return weakref[hashed].v
-        else:
-            logger.debug(f'{name} memoization miss for id (of %s): %s',
-                         len(weakref), hashed)
-    except:
-        logger.debug(f'Failed to hash {name} kwargs', exc_info=True)
-        pass
-    
-    if memoize and (hashed is not None):
-        w = WeakValueWrapper(g)
-        cache_coercion(hashed, w)
-        weakref[hashed] = w
-    return False
 
 
 def make_iframe(url, height, extra_html="", override_html_style=None):
@@ -187,7 +185,7 @@ def make_iframe(url, height, extra_html="", override_html_style=None):
     return iframe + scrollbug_workaround
 
 
-def fingerprint():
+def fingerprint() -> str:
     md5 = hashlib.md5()
     # Hostname, OS, CPU, MAC,
     data = [p.node(), p.system(), p.machine(), str(uuid.getnode())]
@@ -275,30 +273,27 @@ def deprecated(message):
     return deprecated_decorator
 
 
-#
-# def inspect_decorator(func, args, kwargs):
-#     import inspect
-#     frame = inspect.currentframe()
-#     args, _, _, values = inspect.getargvalues(frame)
-#     func_name = inspect.getframeinfo(frame)[2]
-#     print(f'function name "{func_name}"')
-#     for i in args:
-#         print("    %s = %s" % (i, values[i]))
-#     return [(i, values[i]) for i in args]
-#
-#
-# # custom decorator
-# def showargs_decorator(func):
-#     import functools
-#     # updates special attributes e.g. __name__, __doc__
-#     @functools.wraps(func)
-#     def wrapper(*args, **kwargs):
-#
-#       # call custom inspection logic
-#       inspect_decorator(func, args, kwargs)
-#
-#       # calls original function
-#       func(*args, **kwargs)
-#
-#     # matches name of inner function
-#     return wrapper
+# #############################################################################
+
+def is_notebook():
+    """Check if running in a notebook"""
+    try:
+        from IPython import get_ipython
+
+        if "IPKernelApp" not in get_ipython().config:  # pragma: no cover
+            raise ImportError("console")
+            return False
+        if "VSCODE_PID" in os.environ:  # pragma: no cover
+            raise ImportError("vscode")
+            return False
+    except:
+        return False
+    else:  # pragma: no cover
+        return True
+    
+    
+def printmd(string, color=None, size=20):
+    """Print markdown string in notebook"""
+    from IPython.display import Markdown, display
+    colorstr = "<span style='color:{};font-weight:200;font-size:{}px'>{}</span>".format(color, size, string)
+    display(Markdown(colorstr))
